Option Explicit

'glynor's CopySidecar.vbs script
'************************************************************************************************************
'PURPOSE: 	Creates a copy of an existing video file's JRSidecar.xml file with a new File Type
'			This allows you to process an existing file externally, and change its type wthout losing
'			any existing tag data.
'PROCESS: 	The script first refreshes the JRSidecar.xml file for the given file, then makes the copy, and
'			then changes the filename and strips out other filetype-specific data out of the new XML file.
'REQUIRES:	MC18 tested (probably works with older versions too)
'			MC must have Sidecar tagging enabled (which is on by default) for this to be functional
'			Options > General > Importing & Tagging > Store tags in external sidecar files: Video
'USAGE:		CopySidecar.xml /file:"M:\Path\to\target\file.ts"
'			Optional Parameters:
'			/verbosity:on - Asks for confirmation before continuing, and provides additional information.
'			/filetype:ext - The extension for the new file type you are going to create.  Defaults to mkv.
'			Input file type can be of any type for which MC will create a JRSidecar.xml file to store tags.
'			Output file type can be of any type, though it should also be a JRSidecar.xml file type.
'EXAMPLES:	CopySidecar.vbs /file:"M:\Video\Recordings\Fresh Recording.ts"
'				Outputs M:\Video\Recordings\Fresh Recording_mkv_JRSideCar.xml
'				No verbose messages unless there is an error.
'			CopySidecar.vbs /file:"M:\Video\Recordings\Fresh Recording.mkv" /filetype:mp4
'				Outputs M:\Video\Recordings\Fresh Recording_mp4_JRSideCar.xml
'				No verbose messages unless there is an error.
'			CopySidecar.vbs /file:"M:\Video\Recordings\Fresh Recording.mpeg" /verbosity:on /filetype:avi
'				Outputs M:\Video\Recordings\Fresh Recording_avi_JRSideCar.xml
'				Provides verbose messages and allows user to cancel operation before continuing.
'RETURNS:	0 if successful, 1 if failed
'************************************************************************************************************

'************************************************************************************************************
'RELEASE NOTES:
'************************************************************************************************************
'v1.0	Initial release
'		Known Problems:
'			The script tries to force MC to refresh the JRSidecar.xml file for the source file before
'			creating the copy (to make sure tag data in the existing sidecar file is fresh).  However,
'			the MJAutomation.SaveToTag() function seems to return TRUE even when the XML is not properly
'			writen (because the disk location is write-locked, for example).  MC continues to try to
'			write the file in the background, but the script thinks it has completed, so it continues.

'			I need JRiver to fix the SaveToTag() function for this error condition to be properly handled.
'************************************************************************************************************

'Info about this script
Dim strMyName
strMyName = "glynor's CopySidecar.vbs"

'Create our handy dandy Shell and FileSystem objects
Dim WshShell, FileSys
set WshShell = WScript.CreateObject("WScript.Shell")
set FileSys = CreateObject("Scripting.FileSystemObject")

'Get command line arguments
Dim colNamedArguments
Dim sourceFile, targetFileType

set colNamedArguments = WScript.Arguments.Named
'Check for "/verbosity:on" argument
Dim strVerbosity
If colNamedArguments.Exists("verbosity") then
	'Assign the value to the strVerbosity variable and check to see if it's valid
	strVerbosity = colNamedArguments.Item("verbosity")
	if not (strVerbosity = "on" or strVerbosity = "off") then
		ReportError strMyName & " ERROR:" & VbCrLf & VbCrLf & "/verbosity option given with invalid value.  Available values are ""on"" or ""off"".  Value given was: " & strVerbosity, 15, 1
	end if
else
	'Set strVerbosity to the default value of OFF
	strVerbosity = "off"
end if

'Check for "/filetype" argument
If colNamedArguments.Exists("filetype") then
	'Assign the value to the targetFileType variable
	targetFileType = colNamedArguments.Item("filetype")
else
	'Set targetFileType to the default value of mkv
	targetFileType = "mkv"
end if

'Check for "/file" argument
If colNamedArguments.Exists("file") then
	'Assign the value to the sourceFile variable
	sourceFile = colNamedArguments.Item("file")
	'Check that the source file exists and get the various filenames we need.
	If FileSys.FileExists(sourceFile) Then
		'Set our source file type
		Dim sourceFileType
		sourceFileType = FileSys.GetExtensionName(sourceFile)

		'Get the source basename and source XML Filename
		Dim targetFolder, sourceBaseName, sourceXMLFile
		targetFolder = FileSys.GetParentFolderName(sourceFile)
		sourceBaseName = FileSys.GetBaseName(sourceFile)
		sourceXMLFile = FileSys.BuildPath(targetFolder, sourceBaseName & "_" & sourceFileType & "_JRSideCar.xml")
		
		'Get the target filename and XML Filename
		Dim targetFile, targetXMLFile
		targetFile = FileSys.BuildPath(targetFolder, sourceBaseName & "." & targetFileType)
		targetXMLFile = FileSys.BuildPath(targetFolder, sourceBaseName &  "_" & targetFileType & "_JRSideCar.xml")
		
		If (strVerbosity = "on") then
			'Ask before we proceed
			Dim intConfirmButton
			intConfirmButton = WshShell.Popup("Source File: " & sourceFile & vbCRLF & "Source XML: " & sourceXMLFile & vbCRLF & vbCRLF & "Target File: " & targetFile & vbCRLF & "Target XML: " & targetXMLFile & vbCRLF & vbCRLF & "CONTINUE?", , , vbYesNo + vbInformation)
			Select Case intConfirmButton
				Case 6
				'Dont Stop Now
				Case 7
				'Cancel and thrown an error
				ReportError strMyName & " ERROR:" & VbCrLf & VbCrLf & "User canceled operation", 0, 1
			End Select
		End If
	Else
		ReportError strMyName & " ERROR:" & VbCrLf & VbCrLf & "Source File does not exist: " & VbCrLf & sourceFile, 0, 1
	End If
else
	ReportError strMyName & " ERROR:" & VbCrLf & VbCrLf & "/file option not given.", 15, 1
End If

'Refresh the source Sidecar File
If FileSys.FileExists(sourceFile) Then
	'Do the Magic
	If blnMCWriteTags(sourceFile) Then
		'Don't Stop Now
	Else
		ReportError strMyName & " ERROR:" & VbCrLf & VbCrLf & "Failed to write tags for File: " & VbCrLf & sourceFile & vbCRLF & vbCRLF & "Source file is likely not in the active MC Database.", 0, 1
	End If
Else
	ReportError strMyName & " ERROR:" & VbCrLf & VbCrLf & "Source File does not exist: " & VbCrLf & sourceFile, 0, 1
End If

'Create the target Sidecar File
If FileSys.FileExists(sourceXMLFile) Then
	FileSys.CopyFile sourceXMLFile, targetXMLFile
Else
	ReportError strMyName & " ERROR:" & VbCrLf & VbCrLf & "Source XML File does not exist: " & VbCrLf & sourceXMLFile, 0, 1
End If

'Modify your new target Sidecar File
Dim xmlDoc
set xmlDoc = CreateObject( "Microsoft.XMLDOM" )
xmlDoc.Async = "False"
If xmlDoc.Load(targetXMLFile) Then
	
	'Change the filename
	Dim filenameNode
	set filenameNode = xmlDoc.SelectSingleNode("/MPL/Item/Field[@Name='Filename']")
	filenameNode.text = targetFile
	
	'Remove the file type specific tags
	'This is bad.  Maybe use a collection and iterate on it instead?
	Dim compressionNode, bitrateNode, bitdepthNode, samplerateNode
	
	set compressionNode = xmlDoc.SelectSingleNode("/MPL/Item/Field[@Name='Compression']")
	If Not compressionNode Is Nothing Then
		compressionNode.parentNode.removeChild(compressionNode)
	End If
	
	set bitrateNode = xmlDoc.SelectSingleNode("/MPL/Item/Field[@Name='Bitrate']")
	If Not bitrateNode Is Nothing Then
		bitrateNode.parentNode.removeChild(bitrateNode)
	End If
	
	set bitdepthNode = xmlDoc.SelectSingleNode("/MPL/Item/Field[@Name='Bit Depth']")
	If Not bitdepthNode Is Nothing Then
		bitdepthNode.parentNode.removeChild(bitdepthNode)
	End If
	
	set samplerateNode = xmlDoc.SelectSingleNode("/MPL/Item/Field[@Name='Sample Rate']")
	If Not samplerateNode Is Nothing Then
		samplerateNode.parentNode.removeChild(samplerateNode)
	End If
	
	'Save the changes
	xmlDoc.Save(targetXMLFile)
	If (strVerbosity = "on") then
		WshShell.Popup "Target XML File Created: " & vbCRLF & targetXMLFile, 15, , vbInformation
	End If
Else
	' The XML file failed to load
	ReportError strMyName & " ERROR LOADING XML FILE:" & VbCrLf & targetXMLFile, 0, 1
End If

WScript.quit 0

Function blnMCWriteTags(sourceFile)
	'This procedure connects to an instance of MC and then forces it to update the tags for the given sourceFile
	'For video files, this causes MC to write a JRSidecar.xml file out to disk
	
	'Connect to MC instance (try running object first)
	Dim objMC
	On Error Resume Next
	Set objMC = GetObject(, "MediaJukebox Application")
	If Err.Number = 429 Then
		On Error Goto 0
		'Then, create a new object
		Set objMC = CreateObject("MediaJukebox Application")
	End If
	
	'Get the File object from MC
	Dim objMCFile
	Set objMCFile = objMC.GetFile(sourceFile)
	
	'Check to see if MC can access the MJFileAutomation.Filename() property (if not, the file is probably not there)
	If objMCFile.Filename() = "" Then
		ReportError "MC ERROR:" & VbCrLf & VbCrLf & "File record not found: " & VbCrLf & sourceFile & vbCRLF & vbCRLF & "Source file is likely not in the active MC Database.", 0, 1
	End If
	
	'The MJFileAutomation.SaveToTag() function doesn't work if a tag isn't dirty
	'So we can make a tag change (and then change it back to what it was) to mark a field as dirty first
	'Use the Comment tag...
	Dim strSaveComment
	strSaveComment = objMCFile.Comment()
	'We have to use a random number in the comment to make absolutely sure it is unique
	Randomize
	objMCFile.Comment() = "write me a sidecar file - " & Rnd
	objMCFile.Comment() = strSaveComment
	'Write the file
	'NOTE: This seems to always return True, even if the tags don't get written correctly.
	blnMCWriteTags = objMCFile.SaveToTag()
	
End Function

Sub ReportError (strErrorMessage, intTimeout, intFatalError)
	'This procedure reports an error message to the user in a popup, with an timeout
	'And can optionally end the script execution with a error code given.
	'Usage:
	'srtErrorMessage - The full text of the error message shown to the user
	'intTimeout - auto-close the error message box after this many second (set to 0 to never close the box)
	'intFatalError - if > 0 this will cause the script to stop executing, and return the given value as an error code
	
	'Create our handy dandy Shell and FileSystem objects
	Dim WshShell
	set WshShell = WScript.CreateObject("WScript.Shell")
	
	WshShell.Popup strErrorMessage, intTimeout, , vbExclamation
	
	'Future Logging capability
	'WshShell.Run "%comspec% /c echo %COMPUTERNAME% : " & strMyName & " failed (invalid /verbosity): %DATE% %TIME% >> " & strLogFile, 0, False
	
	If intFatalError > 0 Then
		WScript.quit intFatalError
	End If
	
End Sub

